/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

import PropTypes from 'prop-types';
import React, {useContext, useMemo, useState} from 'react';

import ViewsContext from '../../ViewsContext';
import {VIEWS_ACTION_TYPES} from '../../viewsReducer';
import Context from './TableContext';

function ContextProvider({children, columnNames}) {
	const [{modifiedFields}, viewsDispatch] = useContext(ViewsContext);

	const [tableWidth, setTableWidth] = useState(null);
	const [draggingColumnName, setDraggingColumnName] = useState(null);
	const [draggingAllowed, setDraggingAllowed] = useState(true);

	const isFixed = useMemo(() => {
		const allRegistered = columnNames.every(
			(name) => !!modifiedFields[name]
		);

		return allRegistered;
	}, [modifiedFields, columnNames]);

	const resizeColumn = (name, width) => {
		if (!isFixed) {
			return;
		}

		const resizedColumn = modifiedFields[name];

		const isColumnReducing = resizedColumn.width > width;

		let totalWidth = 0;

		Object.values(modifiedFields).forEach((fieldAttributes) => {
			totalWidth += fieldAttributes.width;
		});

		const nextColumnName = columnNames[columnNames.indexOf(name) + 1];

		const nextColumn = modifiedFields[nextColumnName];

		const columnsAreShorterThanContainer = totalWidth < tableWidth;

		if (
			(isColumnReducing &&
				columnsAreShorterThanContainer &&
				!nextColumn?.resizable) ||
			width < 40
		) {
			setDraggingAllowed(false);

			return;
		}

		setDraggingAllowed(true);

		if (isColumnReducing && columnsAreShorterThanContainer) {
			viewsDispatch({
				type: VIEWS_ACTION_TYPES.UPDATE_FIELD,
				value: {
					name: nextColumnName,
					width: nextColumn.width + resizedColumn.width - width,
				},
			});
		}

		viewsDispatch({
			type: VIEWS_ACTION_TYPES.UPDATE_FIELD,
			value: {
				name,
				width,
			},
		});
	};

	return (
		<Context.Provider
			value={{
				columnNames,
				draggingAllowed,
				draggingColumnName,
				isFixed,
				resizeColumn,
				tableWidth,
				updateDraggingAllowed: setDraggingAllowed,
				updateDraggingColumnName: setDraggingColumnName,
				updateTableWidth: setTableWidth,
			}}
		>
			{children}
		</Context.Provider>
	);
}

ContextProvider.defaultProps = {
	columnNames: [],
};

ContextProvider.propTypes = {
	columnNames: PropTypes.arrayOf(PropTypes.string),
};

export default ContextProvider;
